use crate::nodes::{Document, Header, Image, Nodes, Text, Link};

pub mod nodes;
pub mod convert;

static SUPPORTED_FORMAT_VERSION: u64 = 0;

pub fn parse(value: &str) -> Result<Document, anyhow::Error> {
    let res: Result<Document, serde_json::Error> = serde_json::from_str(value);

    match res {
        Ok(res) => {
            if res.version > SUPPORTED_FORMAT_VERSION {
                Err(anyhow::anyhow!("Unsupported document version: {}", res.version))
            } else {
                Ok(res)
            }
        }
        Err(res) => {
            Err(anyhow::Error::from(res))
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    
    #[test]
    fn markdown_to_html() {
        let test_md = r#"# Hi!   

This is a *test* of **[Markdown](https://en.wikipedia.org/wiki/Markdown)** to **[Iresine](https://forge.gaycatgirl.sex/evie/iresine) IL** and then to **[HTML](https://en.wikipedia.org/wiki/HTML)**.

---

### free penguin pictures below!

[![cool penguin image](https://upload.wikimedia.org/wikipedia/commons/b/be/SGI-2016-South_Georgia_%28Fortuna_Bay%29–King_penguin_%28Aptenodytes_patagonicus%29_04.jpg)](https://commons.wikimedia.org/wiki/File:SGI-2016-South_Georgia_(Fortuna_Bay)–King_penguin_(Aptenodytes_patagonicus)_04.jpg)
        "#;
        
        let test_html = r#"<h1>Hi!</h1><p>This is a <em>test</em> of <strong><a href="https://en.wikipedia.org/wiki/Markdown">Markdown</a></strong> to <strong><a href="https://forge.gaycatgirl.sex/evie/iresine">Iresine</a> IL</strong> and then to <strong><a href="https://en.wikipedia.org/wiki/HTML">HTML</a></strong>.</p><hr /><h3>free penguin pictures below!</h3><p><a href="https://commons.wikimedia.org/wiki/File:SGI-2016-South_Georgia_(Fortuna_Bay)–King_penguin_(Aptenodytes_patagonicus)_04.jpg"><img src="https://upload.wikimedia.org/wikipedia/commons/b/be/SGI-2016-South_Georgia_%28Fortuna_Bay%29–King_penguin_%28Aptenodytes_patagonicus%29_04.jpg" /></a></p>"#;
        
        let result = convert::markdown::from_markdown(test_md);

        assert!(result.is_ok());
        
        let html = result.unwrap().to_html();
        
        assert!(html.is_ok());
        assert_eq!(test_html, html.unwrap());
    }
    
    #[test]
    fn test_markdown_conversion() {
        let test_doc = Document {
            version: 0,
            value: vec![
                Nodes::Header(Header {
                    level: "1".to_string(),
                    value: vec![Nodes::Literal("Hi!".to_string())],
                }),
                Nodes::Text(Text {
                    value: vec![
                        Nodes::Literal("This is a test of ".to_string()),
                        Nodes::Link(Link {
                            url: "https://en.wikipedia.org/wiki/Markdown".to_string(),
                            value: vec![
                                Nodes::Literal("Markdown".to_string())
                            ],
                        }),
                        Nodes::Literal(" to ".to_string()),
                        Nodes::Link(Link {
                            url: "https://forge.gaycatgirl.sex/evie/iresine".to_string(),
                            value: vec![
                                Nodes::Literal("Iresine".to_string())
                            ],
                        }),
                        Nodes::Literal(" IL.".to_string())
                    ],
                })
            ],
        };
        
        let test_md = r#"# Hi!   

This is a test of [Markdown](https://en.wikipedia.org/wiki/Markdown) to [Iresine](https://forge.gaycatgirl.sex/evie/iresine) IL.
        "#;
        
        let result = convert::markdown::from_markdown(test_md);

        assert!(result.is_ok());
        assert_eq!(test_doc, result.unwrap());
    }
    
    #[test]
    fn test_string_parse() {
        let test_doc = Document {
            version: 0,
            value: vec![
                Nodes::Header(Header {
                    level: "1".to_string(),
                    value: vec![Nodes::Literal("Hi!".to_string())],
                }),
                Nodes::Image(Image {
                    url: "https://upload.wikimedia.org/wikipedia/commons/7/70/Example.png".to_string(),
                    alt: Some("Example Image".to_string()),
                    blur_hash: Some("L6R{-$oI^bx@O_WB?DtSm$ozafV@".to_string()),
                    width: None,
                    height: None,
                    value: vec![
                        Nodes::Text(Text {
                            value: vec![Nodes::Literal("Hello, world!".to_string())],
                        })
                    ],
                }),
                Nodes::Divider,
                Nodes::Text(Text {
                    value: vec![
                        Nodes::Literal("This is a test of the ".to_string()),
                        Nodes::Link(Link {
                            url: "https://forge.gaycatgirl.sex/evie/iresine".to_string(),
                            value: vec![
                                Nodes::Literal("Iresine".to_string())
                            ],
                        }),
                        Nodes::Literal(" IL.".to_string())
                    ],
                })
            ],
        };

        let test_str = r#"
            {
              "version": 0,
              "value": [
                {
                  "Header": {
                    "level": "1",
                    "value": [
                      {
                        "Literal": "Hi!"
                      }
                    ]
                  }
                },
                {
                  "Image": {
                    "url": "https://upload.wikimedia.org/wikipedia/commons/7/70/Example.png",
                    "alt": "Example Image",
                    "blur_hash": "L6R{-$oI^bx@O_WB?DtSm$ozafV@",
                    "width": null,
                    "height": null,
                    "value": [
                      {
                        "Text": {
                          "value": [
                            {
                              "Literal": "Hello, world!"
                            }
                          ]
                        }
                      }
                    ]
                  }
                },
                "Divider",
                {
                  "Text": {
                    "value": [
                      {
                        "Literal": "This is a test of the "
                      },
                      {
                        "Link": {
                          "url": "https://forge.gaycatgirl.sex/evie/iresine",
                          "value": [
                            {
                              "Literal": "Iresine"
                            }
                          ]
                        }
                      },
                      {
                        "Literal": " IL."
                      }
                    ]
                  }
                }
              ]
            }
        "#;


        let result: Result<Document, anyhow::Error> = parse(test_str);

        assert!(result.is_ok());
        assert_eq!(test_doc, result.unwrap());
    }
}
